@@ -26,11 +26,6 @@ public:
     {
     }
 
-    ~Dpms()
-    {
-        release();
-    }
-
     bool isSupported() const
     {
         return m_supported;
@@ -95,19 +90,13 @@ public:
 
             const auto screens = qGuiApp->screens();
             for (QScreen *screen : screens) {
-                addScreen(screen);
+                QPlatformNativeInterface *native = qGuiApp->platformNativeInterface();
+                wl_output *output = reinterpret_cast<wl_output *>(native->nativeResourceForScreen(QByteArrayLiteral("output"), screen));
+                m_dpmsPerScreen[screen] = new Dpms(get(output), m_dpms, screen);
             }
-            connect(qGuiApp, &QGuiApplication::screenAdded, this, &DpmsManager::addScreen);
-            connect(qGuiApp, &QGuiApplication::screenRemoved, this, [this](QScreen *screen) {
-                delete m_dpmsPerScreen.take(screen);
-            });
             m_dpms->setSupported(hasDpms);
         });
     }
-    ~DpmsManager()
-    {
-        qDeleteAll(m_dpmsPerScreen);
-    }
 
     Dpms *fetch(QScreen *screen)
     {
@@ -115,22 +104,6 @@ public:
     }
 
 private:
-    void addScreen(QScreen *screen)
-    {
-        // We can't rely on checking the wl_output being null yet
-        // https://codereview.qt-project.org/c/qt/qtwayland/+/464669
-        const bool fake = screen->geometry().isEmpty() || screen->name().isEmpty();
-        if (fake) {
-            return;
-        }
-
-        QPlatformNativeInterface *native = qGuiApp->platformNativeInterface();
-        wl_output *output = reinterpret_cast<wl_output *>(native->nativeResourceForScreen(QByteArrayLiteral("output"), screen));
-        if (output) {
-            m_dpmsPerScreen[screen] = new Dpms(get(output), m_dpms, screen);
-        }
-    }
-
     WaylandDpmsHelper *const m_dpms;
     QHash<QScreen *, Dpms *> m_dpmsPerScreen;
 };
@@ -161,7 +134,7 @@ void WaylandDpmsHelper::trigger(KScreen::Dpms::Mode mode, const QList<QScreen *>
     case KScreen::Dpms::Toggle: {
         for (QScreen *screen : screens) {
             auto dpms = m_dpmsManager->fetch(screen);
-            if (!dpms || !dpms->isSupported()) {
+            if (!dpms->isSupported()) {
                 qCDebug(KSCREEN_DPMS) << "screen does not provide dpms" << screen;
                 continue;
             }
@@ -188,17 +161,12 @@ void WaylandDpmsHelper::trigger(KScreen::Dpms::Mode mode, const QList<QScreen *>
     }
     for (auto screen : screens) {
         auto dpms = m_dpmsManager->fetch(screen);
-        if (dpms) {
-            dpms->set(level);
-        }
+        dpms->set(level);
     }
     setHasPendingChanges(false);
 }
 
-void WaylandDpmsHelper::blockUntilSupported()
+void WaylandDpmsHelper::blockUntilSupported() const
 {
     QMetaObject::invokeMethod(m_dpmsManager, "addRegistryListener");
-    if (!m_dpmsManager->isActive()) {
-        setSupported(false);
-    }
 }
